require 'spec_helper'

describe BranchableCDNAssets::Config do

  let(:data) do
    {
      production_branch: 'production',
      default_env: 'default',
      dir: 'cdn_dir',
      environments: {
        production: {
          host: 'production',
          url:  'http://production.com',
          root: '/var/www/production',
          invalidator: {
            cloudfront: {
              access_key:      'FooBar',
              secret_key:      'SecretBaz',
              distribution_id: 'Distro',
              path_prefix:     '/'
            }
          }
        },
        test: {
          host: 'test',
          url:  'http://test.com',
          root: '/var/www/test'
        },
        staging: {
          host: 'staging',
          url:  'http://staging.com',
          root: '/var/www/staging'
        },
        default: {
          host: 'default',
          url:  'http://default.com',
          root: '/var/www/default'
        }
      }
    }
  end

  let(:yaml) do
    "foo: bar\n" +
    "baz: wu\n"
  end

  let(:yaml_erb) do
    "foo: <%= 'bar' %>"
  end

  describe "#raw_data" do
    context "when given a hash" do
      it "uses hash data as is" do
        expect( described_class.new( data ).raw_data ).to eq data
      end
      it "symbolizes data's keys if they are strings" do
        expect( described_class.new( {'foo'=>'foo', 'bar' => { 'baz' => 'baz' }} ).raw_data ).to eq foo: 'foo', bar: { baz: 'baz' }
      end
    end
    context "when given a path" do
      it "reads yaml data in config" do
        allow(File).to receive(:exists?).with('yaml').and_return(true)
        allow(IO).to receive(:read).with('yaml').and_return(yaml)

        expect( described_class.new('yaml').raw_data ).to eq foo: 'bar', baz: 'wu'
      end
      it "reads erb in yaml" do
        allow(File).to receive(:exists?).with('yaml_erb').and_return(true)
        allow(IO).to receive(:read).with('yaml_erb').and_return(yaml_erb)

        expect( described_class.new('yaml_erb').raw_data ).to eq foo: 'bar'
      end
      it "raises exception if file not found" do
        allow(File).to receive(:exists?).with('yaml').and_return(false)

        expect{
          described_class.new('yaml')
        }.to raise_error
      end
    end
  end

  describe "#branch" do
    it "defaults to the Asgit.current_branch" do
      allow( Asgit ).to receive(:current_branch).and_return('foo_bar')
      expect( described_class.new( data ).branch ).to eq 'foo_bar'
    end
    it "can be overriden by passing in a branch" do
      expect( described_class.new( data, 'baz' ).branch ).to eq 'baz'
    end
  end

  describe "#production_branch" do
    it "defaults to 'master' if none is passed" do
      expect( described_class.new( data.without_key(:production_branch) ).production_branch ).to eq 'master'
    end
    it "can be set with data" do
      expect( described_class.new( data ).production_branch ).to eq 'production'
    end
  end

  describe "#invalidator" do

    it "returns instance of given invalidator class" do
      expect( described_class.new( data, 'production' ).invalidator ).to be_a BranchableCDNAssets::Invalidator::Cloudfront
    end

    it "returns false if no invalidator given" do
      subject_data = data.dup
      subject_data[:environments][:production].delete(:invalidator)

      expect( described_class.new( subject_data, 'production' ).invalidator ).to be false
    end

    it "raises error if given an unregistered invalidator" do
      subject_data = data.dup
      subject_data[:environments][:production][:invalidator] = { foo: 'bar' }

      expect{
        described_class.new(subject_data, 'production').invalidator
      }.to raise_error
    end
  end

  describe "#cdn_dir" do
    it "is set to the passed value" do
      expect( described_class.new( data ).cdn_dir ).to eq 'cdn_dir'
    end

    it "falls back to 'cdn' if no value is passed" do
      expect( described_class.new( data.without_key(:dir) ).cdn_dir ).to eq 'cdn'
    end
  end

  describe "#file_filter" do
    it "returns nil if not set" do
      expect( described_class.new( data ).file_filter ).to eq nil
    end

    it "raises an argument error if not a proc with one argument" do
      expect{
        described_class.new( data.merge(file_filter: "foo") )
      }.to raise_error ArgumentError

      expect{
        described_class.new( data.merge(file_filter: lambda { |x,y| } ) )
      }.to raise_error ArgumentError
    end

    it "returns the given proc" do
      filter_proc = lambda { |file_path| file_path }
      expect( described_class.new( data.merge(file_filter: filter_proc) ).file_filter ).to eq filter_proc
    end
  end

  describe "#env" do
    context "when the current branch matches production_branch" do
      before :each do
        allow( Asgit ).to receive(:current_branch).and_return('production')
      end
      it "is :production" do
        expect( described_class.new( data ).env ).to eq :production
      end
    end
    context "when on a branch with a matching environment key" do
      before :each do
        allow( Asgit ).to receive(:current_branch).and_return('test')
      end
      it "matchs the current_branch" do
        expect( described_class.new( data ).env ).to eq :test
      end
    end
    context "when on a branch that doesn't match an env key" do
      before :each do
        allow( Asgit ).to receive(:current_branch).and_return('random')
      end
      it "sets env to the set default_env" do
        expect( described_class.new( data ).env ).to eq :default
      end
      it "falls back to 'staging' if no default env given" do
        expect( described_class.new( data.without_key(:default_env) ).env ).to eq :staging
      end
    end
  end

  describe "#environments" do
    it "returns data namespaced with branch" do
      production = described_class.new( data, 'production' )
      default    = described_class.new( data, 'foo_bar' )

      expect( production.root ).to eq '/var/www/production'
      expect( production.host ).to eq 'production'
      expect( production.url  ).to eq 'http://production.com'

      expect( default.root ).to eq '/var/www/default/foo_bar/'
      expect( default.host ).to eq 'default'
      expect( default.url  ).to eq 'http://default.com/foo_bar'
    end
  end

end
