require 'spec_helper'

describe BranchableCDNAssets::FileManager do

  before :each do
    @config_data = {
      env: 'default',
      cdn_dir: 'cdn',
      dir_permissions: 755,
      file_permissions: 644,
      rsync_flags: '-aviz',
      remotes: [
        BranchableCDNAssets::Config::Remote.new( 'host', 'root', 'url', 755, 644, '-aviz' )
      ],
      environments: {
        staging: {
          host: 'host',
          root: 'root',
          url: 'http://foo.com'
        },
        production: {
          host: 'host',
          root: 'root',
          url: 'http://foo.com',
          invalidator: {
            cloudfront: {
              distribution_id: '',
              access_key: '',
              secret_key: '',
              path_prefix: '/prefix'
            }
          }
        }
      }
    }
    @config = BranchableCDNAssets::Config.new @config_data, 'master'
    allow( BranchableCDNAssets::Config ).to receive(:new).and_return( @config )

    @file_list = [
      'filename_one',
      'filename_two',
      'directory/dir_file'
    ]

    allow( Dir).to receive(:[]).with(anything()).and_call_original
    allow( Dir).to receive(:[]).with( "#{Dir.pwd}/cdn/**/*" ).and_return( @file_list.map { |f| File.join( Dir.pwd, 'cdn', f ) } )

    @manifest = instance_double("BranchableCDNAssets::Manifest", files: @file_list )
    allow( BranchableCDNAssets::Manifest ).to receive(:new).and_return( @manifest )

    @cloudfront = BranchableCDNAssets::Invalidator::Cloudfront.new(@config_data[:environments][:production][:invalidator][:cloudfront])
    allow( BranchableCDNAssets::Invalidator::Cloudfront ).to receive(:new).and_return( @cloudfront )
    allow( @cloudfront ).to receive(:invalidate_files)

    @tempfile = instance_double( "Tempfile",
      unlink: nil,
      path: 'tempfile_path',
      write: nil,
      close: nil
    )
    allow( Tempfile ).to receive(:new).and_return( @tempfile )

    @success_response = instance_double("HereOrThere::Response",
      success?: true,
      stderr: "",
      stdout: ">f+++++++++ filename_one\n" +
              ">f+++++++++ filename_two\n" +
              ">d+++++++++ directory/\n" +
              ">f+++++++++ directory/dir_file\n"
    )

    @error_response = instance_double("HereOrThere::Response",
      success?: false,
      stderr: "stderr",
      stdout: ""
    )
  end

  describe '#config' do
    it "returns the provided config" do
      files = described_class.new( @config )
      expect( files.config ).to eq @config
    end
  end

  describe '#root' do
    it "returns the provided root from config" do
      files = described_class.new( @config )
      expect( files.root ).to eq 'cdn'
    end
  end

  describe '#manifest' do
    it "returns a manifest with the environment manifest path" do
      expect( BranchableCDNAssets::Manifest ).to receive(:new).with( "cdn/#{@config.branch}.manifest" )
      files = described_class.new( @config )
      expect( files.manifest ).to eq @manifest
    end
    it "handles 'bad' filenames and slugs them" do
      @config_data = {
        env: 'default',
        branch: 'branch/oh-noes',
        cdn_dir: 'cdn',
        host: 'host',
        root: 'root',
        invalidator: {
          cloudfront: {
            path_prefix: '/prefix'
          }
        }
      }
      @config = instance_double( "BranchableCDNAssets::Config", @config_data )
      allow( BranchableCDNAssets::Config ).to receive(:new).and_return( @config )

      expect( BranchableCDNAssets::Manifest ).to receive(:new).with("cdn/branch_oh-noes.manifest")
      files = described_class.new( @config )
    end
  end

  describe "#branch" do
    it "is defaults to the config.branch" do
      expect( described_class.new(@config).branch ).to eq @config.branch
    end
    it "can be overriden by passed arg" do
      expect( described_class.new(@config, 'foo').branch ).to eq 'foo'
    end
  end

  describe '#list' do

    let(:remote_files) { ['filename_one', 'remote_one', 'remote_two'] }
    let(:local_files)  { @file_list }

    before :each do
      allow( @manifest ).to receive(:files).and_return(remote_files)
    end

    it "returns remote files when arg is :remote" do
      expect( described_class.new(@config).list(:remote) ).to match_array remote_files
    end

    it "returns local files when arg is :local" do
      expect( described_class.new(@config).list(:local) ).to match_array local_files
    end

    it "uses the given file_filter to filter files when :local" do
      file_filter = lambda { |file_path| return !file_path.match('directory') }
      @config = instance_double( "BranchableCDNAssets::Config",
                                 @config_data.merge( file_filter: file_filter,
                                                     branch: 'master' ) )
      allow( BranchableCDNAssets::Config ).to receive(:new).and_return( @config )

      expect( described_class.new( @config ).list(:local) ).to match_array local_files - ['directory/dir_file']
    end

    it "returns both local and remote files when :all" do
      expect( described_class.new(@config).list() ).to match_array remote_files + local_files
    end

    it "returns intersecting files whe :both" do
      expect( described_class.new(@config).list(:both) ).to match_array ['filename_one']
    end

  end

  describe "#pull!" do

    context "when pull is successful" do
      before :each do
        allow( BranchableCDNAssets::Shell ).to receive(:run_local).and_return( @success_response )
        allow( @manifest ).to receive(:remove_files)
        allow( @manifest ).to receive(:update_source_file!)
      end

      it "returns list of pulled files" do
        allow( BranchableCDNAssets::Shell ).to receive(:run_remote)
                                           .with("find root -type f", hostname: "host")
                                           .and_return(@success_response)
        expect( described_class.new(@config).pull! ).to match_array @file_list
      end

      it "removes files & updates manifest" do
        allow( BranchableCDNAssets::Shell ).to receive(:run_remote)
                                           .with("find root -type f", hostname: "host")
                                           .and_return(@success_response)
        expect( @manifest ).to receive(:remove_files).with(@file_list)
        expect( @manifest ).to receive(:update_source_file!)
        described_class.new(@config).pull!
      end
    end

    context "when pull returns error" do
      it "raises an error" do
        allow( BranchableCDNAssets::Shell ).to receive(:run_local).and_return( @error_response )

        expect{
          described_class.new(@config).pull!
        }.to raise_error
      end
    end
  end

  describe "#push!" do

    before :each do
      allow( BranchableCDNAssets::Shell ).to receive(:run_remote)
      allow( BranchableCDNAssets::Shell ).to receive(:run_local).and_return( @success_response )
      allow( @manifest ).to receive(:merge_files)
      allow( @manifest ).to receive(:update_source_file!)
    end

    context "when setup_remote is successful" do
      before :each do
        allow( BranchableCDNAssets::Shell ).to receive(:run_remote)
                                           .with('mkdir -p -m 755 root', hostname: 'host')
                                           .and_return(@success_response)
      end

      it "sets up remote" do
        expect( BranchableCDNAssets::Shell ).to receive(:run_remote)
                                            .with('mkdir -p -m 755 root', hostname: 'host')
                                            .and_return(@success_response)
        described_class.new(@config).push!
      end

      it "returns an array of pushed files" do
        expect( described_class.new(@config).push! ).to match_array @file_list
      end

      it "adds files and updates the manifest" do
        expect( @manifest ).to receive(:merge_files).with(@file_list)
        expect( @manifest ).to receive(:update_source_file!)

        described_class.new(@config).push!
      end

      context "when env is production" do

        before :each do
          allow( @config ).to receive(:env).and_return(:production)
        end

        it "invalidates intersecting files" do
          expect( @cloudfront ).to receive(:invalidate_files).with( @file_list.map { |f| "/prefix/#{f}"} )
          described_class.new(@config).push!
        end
      end
    end

    context "when setup_remote fails" do
      it "aborts and prints error" do
        allow( BranchableCDNAssets::Shell ).to receive(:run_remote).and_return( @error_response )
        expect {
          described_class.new(@config).push!
        }.to raise_error
      end
    end
  end

  describe "#prune!" do

    let(:remote_files) { ['filename_one', 'remote_one', 'remote_two'] }
    let(:local_files)  { @file_list }

    before :each do
      allow( @manifest ).to receive(:files).and_return(remote_files)
    end

    context "when confirm removal" do
      after :each do
        `rm -rf cdn`
      end

      it "removes files that are local and on remote" do
        expect( BranchableCDNAssets::Shell ).to receive(:run_local).with("rm cdn/filename_one")
        release_stdin 'y' do
          described_class.new(@config).prune!
        end
      end

      it "removes empty directories" do
        files = described_class.new(@config)
        allow( Dir).to receive(:[]).with( "#{Dir.pwd}/cdn/**/*" ).and_call_original

        `mkdir -p cdn/empty_dir`
        `mkdir -p cdn/dir/empty_dir`
        `mkdir -p cdn/foo`
        `touch cdn/bar`
        `touch cdn/foo/baz`

        release_stdin 'y' do
          files.prune!
        end

        expect( HereOrThere::Local.new.run("ls cdn").stdout ).to eq "bar\nfoo\n"
      end
    end

    context "when removal aborted" do
      it "does not remove files and exits" do
        expect( BranchableCDNAssets::Shell ).not_to receive(:run_local).with(/rm\s[\w\/\.]+$/)

        expect{
          release_stdin 'n' do
            described_class.new(@config).prune!
          end
        }.to raise_error SystemExit
      end
    end
  end

  describe "#move_to_production" do
    before :each do
      @orig_cdn   = described_class.new(@config)
      @branch_cdn = instance_double("BranchableCDNAssets::FileManager", manifest: @manifest )
      allow( BranchableCDNAssets::Shell ).to receive(:run_local).and_return(@success_response)
      allow( described_class ).to receive(:new).and_call_original
      allow( @config ).to receive(:raw_data)
      allow( described_class ).to receive(:new).with( @config ).and_return( @branch_cdn )
    end

    it "pulls from remote branch & pushes on current branch" do
      expect( @branch_cdn ).to receive(:pull!).and_return([])
      expect( @orig_cdn ).to   receive(:push!).and_return([])
      @orig_cdn.move_to_production('test')
    end
  end

end
