require 'spec_helper'

describe BranchableCDNAssets::Manifest do

  describe "#source_file" do
    it "returns the path initialized with" do
      expect( BranchableCDNAssets::Manifest.new('/this/path').source_file ).to eq '/this/path'
    end
  end

  describe "#file_set" do
    let(:source_file) { 'manifest.manifest' }

    context "when source_file exists" do

      before :each do
        allow( File ).to receive(:exists?).with(source_file).and_return(true)
        allow( IO ).to receive(:read).with(source_file).and_return("foo\nbar")
      end

      it "returns a Set" do
        expect( BranchableCDNAssets::Manifest.new(source_file).file_set ).to be_a Set
      end

      it "contains the files sep by newline in the manifest" do
        files = BranchableCDNAssets::Manifest.new(source_file).file_set
        expect( files.include?('foo') ).to be_truthy
      end
    end

    context "when source_file doesn't exist" do

      before :each do
        allow( File ).to receive(:exists?).with(source_file).and_return(false)
      end

      it "returns an empty set" do
        expect( BranchableCDNAssets::Manifest.new(source_file).file_set ).to be_empty
      end
    end

  end

  describe "#files" do

    let(:source_file) { 'manifest.manifest' }
    let(:manifest)    { BranchableCDNAssets::Manifest.new(source_file) }

    before :each do
      allow( File ).to receive(:exists?).with(source_file).and_return(true)
      allow( IO ).to receive(:read).with(source_file).and_return("foo\nbar")
    end

    it "returns an Array" do
      expect( manifest.files ).to be_a Array
    end

    it "matches the file_set Set" do
      expect( manifest.files ).to match_array manifest.file_set.to_a
    end

    it "handles windows like file systems" do
      allow( IO ).to receive(:read).with(source_file).and_return("foo\r\nbar")
      expect( manifest.files ).to match_array ["foo", "bar"]
    end

  end

  describe "#merge_files" do

    it "adds arrays of new files" do
      manifest = BranchableCDNAssets::Manifest.new('foo')
      expect{ manifest.merge_files(["hello","world"]) }.to change{manifest.files.count}.by(2)
      expect( manifest.files.include?("hello") ).to be_truthy
    end

    it "adds single new files" do
      manifest = BranchableCDNAssets::Manifest.new('foo')
      expect{ manifest.merge_files("hello") }.to change{manifest.files.count}.by(1)
      expect( manifest.files.include?("hello") ).to be_truthy
    end

    it "doesn't add duplicate files" do
      manifest = BranchableCDNAssets::Manifest.new('foo')
      manifest.merge_files("hello")

      expect{ manifest.merge_files("hello") }.not_to change{manifest.files.count}
    end

  end

  describe "#remove_files" do

    before :each do
      @manifest = BranchableCDNAssets::Manifest.new('foo')
      @manifest.merge_files(["hello","world"])
    end

    it "removes arrays of files" do
      expect{ @manifest.remove_files(["hello","world"]) }.to change{@manifest.files.count}.by(-2)
    end

    it "removes single files" do
      expect{ @manifest.remove_files("hello") }.to change{@manifest.files.count}.by(-1)
    end

  end

  describe "#update_source_file!" do

    context "when files is empty" do
      it "removes the source file and returns" do
        manifest = BranchableCDNAssets::Manifest.new('foo')

        expect( File ).not_to receive(:open)
        allow( File ).to receive(:exist?).with('foo').and_return(true)
        expect( File ).to receive(:delete).with('foo')

        manifest.update_source_file!
      end
    end

    context "when files has contents" do

      before :each do
        @source_file = Tempfile.new('foo')
        @source_path = @source_file.path
        @manifest    = BranchableCDNAssets::Manifest.new(@source_path)
        @manifest.merge_files(["hello","world"])
      end

      it "writes contents to the source_file" do
        expect( File ).to receive(:open).with(@source_path, 'w').and_yield(@source_file)
        expect( @source_file ).to receive(:write).with("hello\nworld")

        @manifest.update_source_file!
      end

    end

  end


end
